using System;
using UnityEngine;
using UnityEngine.Experimental.Rendering;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Linq;
using MVR;
using MeshVR;
using MVR.FileManagementSecure;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using GPUTools.Cloth.Scripts;
using Unity.Collections;
using AssetBundles;
using System.Diagnostics;
using UnityEngine.Events;

namespace Stopper {
	public class WearableCUA : MVRScript {
        Transform originalParent;

        Atom containingAtom;
        Atom parentAtom;
        DAZBone parentBone;

		public override void Init() {
            containingAtom = base.containingAtom;

             JSONStorableStringChooser personChooser = new JSONStorableStringChooser(
            "Person", 
            GetPeople(),
            null, 
            "Person", 
            (string person) => {
                parentAtom = SuperController.singleton.GetAtomByUid(person);
            });
			var personChooserPopup = CreateScrollablePopup(personChooser, false);
            personChooserPopup.popup.onOpenPopupHandlers += () => {
				personChooser.choices = GetPeople();
			};
            RegisterStringChooser(personChooser);

            JSONStorableStringChooser parentBoneChooser = new JSONStorableStringChooser(
            "Parent Bone", 
            new List<string>(),
            null, 
            "Parent Bone", 
            (string boneName) => {
                if(parentAtom != null) {
                    parentBone = parentAtom.GetComponentsInChildren<DAZBone>(true).First((bone) => bone.name == boneName);

                    Parent();
                }
            });
			var parentBoneChooserPopup = CreateFilterablePopup(parentBoneChooser, false);
            parentBoneChooserPopup.popup.onOpenPopupHandlers += () => {
                if(parentAtom != null) {
				    parentBoneChooser.choices = GetBones(parentAtom);
                } else {
                    parentBoneChooser.choices = new List<string>();
                }
			};
            RegisterStringChooser(parentBoneChooser);

            OnEnable();
		}

        private List<string> GetPeople(){
            return SuperController.singleton.GetAtoms().Where((a) => a.GetComponentInChildren<DAZBones>(true) != null).Select((a) => a.uid).ToList();
        }

        private List<string> GetBones(Atom person) {
            List<string> boneNames = new List<string>();
            foreach(var bone in person.GetComponentsInChildren<DAZBone>(true)){
                boneNames.Add(bone.name);
            }
            return boneNames;
        }

        private void Parent() {
            if(parentBone != null){
                if(originalParent == null) {
                    originalParent = containingAtom.transform.parent;
                }
                containingAtom.transform.parent = parentBone.transform;
            }
        }

        private void Unparent() {
            containingAtom.transform.parent = originalParent;
            originalParent = null;
        }
	
        public UIDynamicSlider AddSlider(JSONStorableFloat storableFloat, UnityAction<float> onChange = null, bool rangeAdjust = true, bool rightSide = false){
            UIDynamicSlider uiSlider = this.CreateSlider(storableFloat, rightSide);
            uiSlider.rangeAdjustEnabled = rangeAdjust;
			uiSlider.valueFormat = "F4";
            RegisterFloat(storableFloat);

            if(onChange != null){
				uiSlider.slider.onValueChanged.AddListener(onChange);
			}
			
			return uiSlider;
		}
        
		private void OnEnable() {
            Parent();
        }
		private void OnDisable() {
            Unparent();
		}

        private void OnDestroy() {
        }
	}
}